@@ -1,107 +1,12 @@
 /* libmondo-fork.c
-   $Id: libmondo-fork.c 1316 2007-04-16 14:50:28Z bruno $
+   $Id: libmondo-fork.c 3060 2012-11-10 04:05:37Z bruno $
 
 - subroutines for handling forking/pthreads/etc.
-
-
-01/20/2006
-- replaced partimagehack with ntfsclone
-
-06/20/2004
-- create fifo /var/log/partimagehack-debug.log and empty it
-  to keep ramdisk from filling up
-
-04/13/2004
-- >= should be <= g_loglevel
-
-11/15/2003
-- changed a few []s to char*s
-  
-10/12
-- rewrote partimagehack handling (multiple fifos, chunks, etc.)
-
-10/11
-- partimagehack now has debug level of N (set in my-stuff.h)
-
-10/08
-- call to partimagehack when restoring will now log errors to /var/log/....log
-
-10/06
-- cleaned up logging a bit
-
-09/30
-- line 735 - missing char* cmd in sprintf()
-
-09/28
-- added run_external_binary_with_percentage_indicator()
-- rewritten eval_call_to_make_ISO()
-
-09/18
-- call mkstemp instead of mktemp
-
-09/13
-- major NTFS hackage
-
-09/12
-- paranoid_system("rm -f /tmp/ *PARTIMAGE*") before calling partimagehack
-
-09/11
-- forward-ported unbroken feed_*_partimage() subroutines
-  from early August 2003
-
-09/08
-- detect & use partimagehack if it exists
-
-09/05
-- finally finished partimagehack hack :)
-
-07/04
-- added subroutines to wrap around partimagehack
-
-04/27
-- don't echo (...res=%d...) at end of log_it()
-  unnecessarily
-- replace newtFinished() and newtInit() with
-  newtSuspend() and newtResume()
-
-04/24
-- added some assert()'s and log_OS_error()'s
-
-04/09
-- cleaned up run_program_and_log_output()
-
-04/07
-- cleaned up code a bit
-- let run_program_and_log_output() accept -1 (only log if _no error_)
-
-01/02/2003
-- in eval_call_to_make_ISO(), append output to MONDO_LOGFILE
-  instead of a temporary stderr text file
-
-12/10
-- patch by Heiko Schlittermann to handle % chars in issue.net
-
-11/18
-- if mkisofs in eval_call_to_make_ISO() returns an error then return it,
-  whether ISO was created or not
-
-10/30
-- if mkisofs in eval_call_to_make_ISO() returns an error then find out if
-  the output (ISO) file has been created; if it has then return 0 anyway
-
-08/01 - 09/30
-- run_program_and_log_output() now takes boolean operator to specify
-  whether it will log its activities in the event of _success_
-- system() now includes 2>/dev/null
-- enlarged some tmp[]'s
-- added run_program_and_log_to_screen() and run_program_and_log_output()
-
-07/24
-- created
 */
 
 
 #include "my-stuff.h"
+#include "mr_mem.h"
 #include "mondostructures.h"
 #include "libmondo-fork.h"
 #include "libmondo-string-EXT.h"
@@ -111,12 +16,14 @@
 #include "lib-common-externs.h"
 
 /*@unused@*/
-//static char cvsid[] = "$Id: libmondo-fork.c 1316 2007-04-16 14:50:28Z bruno $";
+//static char cvsid[] = "$Id: libmondo-fork.c 3060 2012-11-10 04:05:37Z bruno $";
 
-extern char *g_tmpfs_mountpt;
 extern t_bkptype g_backup_media_type;
 extern bool g_text_mode;
 extern char *MONDO_LOGFILE;
+
+/* Reference to global bkpinfo */
+extern struct s_bkpinfo *bkpinfo;
 pid_t g_buffer_pid = 0;
 
 
@@ -129,8 +36,9 @@ pid_t g_buffer_pid = 0;
 char *call_program_and_get_last_line_of_output(char *call)
 {
 	/*@ buffers ***************************************************** */
-	static char result[512];
+	static char result[MAX_STR_LEN];
 	char *tmp;
+	char *p;
 
 	/*@ pointers **************************************************** */
 	FILE *fin;
@@ -144,8 +52,8 @@ char *call_program_and_get_last_line_of_output(char *call)
 
 	assert_string_is_neither_NULL_nor_zerolength(call);
 	if ((fin = popen(call, "r"))) {
-		for (fgets(tmp, MAX_STR_LEN, fin); !feof(fin);
-			 fgets(tmp, MAX_STR_LEN, fin)) {
+		for (p = fgets(tmp, MAX_STR_LEN, fin); !feof(fin) && (p != NULL);
+			 p = fgets(tmp, MAX_STR_LEN, fin)) {
 			if (strlen(tmp) > 1) {
 				strcpy(result, tmp);
 			}
@@ -171,7 +79,6 @@ char *call_program_and_get_last_line_of_output(char *call)
  * @param bkpinfo The backup information structure. Fields used:
  * - @c bkpinfo->manual_cd_tray
  * - @c bkpinfo->backup_media_type
- * - @c bkpinfo->please_dont_eject_when_restoring
  * @param basic_call The call to mkisofs. May contain tokens that will be resolved to actual data. The tokens are:
  * - @c _ISO_ will become the ISO file (@p isofile)
  * - @c _CD#_ becomes the CD number (@p cd_no)
@@ -184,8 +91,7 @@ char *call_program_and_get_last_line_of_output(char *call)
  * @bug @p logstub is unused.
  */
 int
-eval_call_to_make_ISO(struct s_bkpinfo *bkpinfo,
-					  char *basic_call, char *isofile,
+eval_call_to_make_ISO(char *basic_call, char *isofile,
 					  int cd_no, char *logstub, char *what_i_am_doing)
 {
 
@@ -197,6 +103,7 @@ eval_call_to_make_ISO(struct s_bkpinfo *bkpinfo,
 	char *midway_call, *ultimate_call, *tmp, *command, *incoming,
 		*old_stderr, *cd_number_str;
 	char *p;
+	char *tmp1 = NULL;
 
 /*@***********   End Variables ***************************************/
 
@@ -225,11 +132,17 @@ eval_call_to_make_ISO(struct s_bkpinfo *bkpinfo,
 	incoming[0] = '\0';
 	old_stderr[0] = '\0';
 
+	if ((bkpinfo->netfs_user) && (strstr(bkpinfo->netfs_proto,"nfs"))) {
+		mr_asprintf(&tmp1, "su - %s -c \"%s\"", bkpinfo->netfs_user, basic_call);
+	} else {
+		mr_asprintf(&tmp1, "%s", basic_call);
+	}
+
 	sprintf(cd_number_str, "%d", cd_no);
-	resolve_naff_tokens(midway_call, basic_call, isofile, "_ISO_");
+	resolve_naff_tokens(midway_call, tmp1, isofile, "_ISO_");
 	resolve_naff_tokens(tmp, midway_call, cd_number_str, "_CD#_");
 	resolve_naff_tokens(ultimate_call, tmp, MONDO_LOGFILE, "_ERR_");
-	log_msg(4, "basic call = '%s'", basic_call);
+	log_msg(4, "basic call = '%s'", tmp1);
 	log_msg(4, "midway_call = '%s'", midway_call);
 	log_msg(4, "tmp = '%s'", tmp);
 	log_msg(4, "ultimate call = '%s'", ultimate_call);
@@ -263,7 +176,7 @@ eval_call_to_make_ISO(struct s_bkpinfo *bkpinfo,
 			newtResume();
 		}
 		if (retval) {
-			log_msg(2, "Basic call '%s' returned an error.", basic_call);
+			log_msg(2, "Basic call '%s' returned an error.", tmp1);
 			popup_and_OK("Press ENTER to continue.");
 			popup_and_OK
 				("mkisofs and/or cdrecord returned an error. CD was not created");
@@ -278,6 +191,7 @@ eval_call_to_make_ISO(struct s_bkpinfo *bkpinfo,
 			(what_i_am_doing, command);
 	}
 
+	mr_free(tmp1);
 	paranoid_free(midway_call);
 	paranoid_free(ultimate_call);
 	paranoid_free(tmp);
@@ -285,13 +199,47 @@ eval_call_to_make_ISO(struct s_bkpinfo *bkpinfo,
 	paranoid_free(incoming);
 	paranoid_free(old_stderr);
 	paranoid_free(cd_number_str);
-/*
-  if (bkpinfo->backup_media_type == dvd && !bkpinfo->please_dont_eject_when_restoring)
-    {
-      log_msg(3, "Ejecting DVD device");
-      eject_device(bkpinfo->media_device);
-    }
-*/
+	return (retval);
+}
+
+
+/**
+ * Call copy of data to create an USB image.
+ * @param bkpinfo The backup information structure. Fields used:
+ * - @c bkpinfo->backup_media_type
+ * @return Exit code of @c copy (0 is success, anything else indicates failure).
+ */
+int
+eval_call_to_make_USB(char *command, char *what_i_am_doing) {
+
+	/*@ int's  *** */
+	int retval = 0;
+
+
+/*@***********   End Variables ***************************************/
+
+	log_msg(3, "Starting");
+	assert(bkpinfo != NULL);
+
+	log_to_screen
+		("Please be patient. Do not be alarmed by on-screen inactivity.");
+	log_msg(4, "Calling open_evalcall_form() with what_i_am_doing='%s'",
+			what_i_am_doing);
+
+	if (!g_text_mode) {
+		newtSuspend();
+	}
+	log_msg(1, "command = '%s'", command);
+	if (!g_text_mode) {
+		retval = run_external_binary_with_percentage_indicator_NEW
+			(what_i_am_doing, command);
+	} else {
+		retval += system(command);
+	}
+	if (!g_text_mode) {
+		newtResume();
+	}
+
 	return (retval);
 }
 
@@ -307,21 +255,19 @@ eval_call_to_make_ISO(struct s_bkpinfo *bkpinfo,
 int run_program_and_log_output(char *program, int debug_level)
 {
 	/*@ buffer ****************************************************** */
-	char callstr[MAX_STR_LEN * 2];
+	char *callstr = NULL;
 	char incoming[MAX_STR_LEN * 2];
-	char tmp[MAX_STR_LEN * 2];
-	char initial_label[MAX_STR_LEN * 2];
+	char *tmp1 = NULL;
 
 	/*@ int ********************************************************* */
 	int res;
-	int i;
-	int len;
 	bool log_if_failure = FALSE;
 	bool log_if_success = FALSE;
 
 	/*@ pointers *************************************************** */
 	FILE *fin;
 	char *p;
+	char *q;
 
 	/*@ end vars *************************************************** */
 
@@ -338,9 +284,7 @@ int run_program_and_log_output(char *program, int debug_level)
 		log_if_success = TRUE;
 		log_if_failure = TRUE;
 	}
-	sprintf(callstr,
-			"%s > /tmp/mondo-run-prog-thing.tmp 2> /tmp/mondo-run-prog-thing.err",
-			program);
+	mr_asprintf(&callstr, "%s > %s/mondo-run-prog-thing.tmp 2> %s/mondo-run-prog-thing.err", program, bkpinfo->tmpdir, bkpinfo->tmpdir);
 	while ((p = strchr(callstr, '\r'))) {
 		*p = ' ';
 	}
@@ -349,37 +293,27 @@ int run_program_and_log_output(char *program, int debug_level)
 	}							/* single '=' is intentional */
 
 
-	len = (int) strlen(program);
-	for (i = 0; i < 35 - len / 2; i++) {
-		tmp[i] = '-';
-	}
-	tmp[i] = '\0';
-	strcat(tmp, " ");
-	strcat(tmp, program);
-	strcat(tmp, " ");
-	for (i = 0; i < 35 - len / 2; i++) {
-		strcat(tmp, "-");
-	}
-	strcpy(initial_label, tmp);
 	res = system(callstr);
 	if (((res == 0) && log_if_success) || ((res != 0) && log_if_failure)) {
 		log_msg(0, "running: %s", callstr);
-		log_msg(0,
-				"--------------------------------start of output-----------------------------");
+		log_msg(0, "--------------------------------start of output-----------------------------");
 	}
-	if (log_if_failure
-		&&
-		system
-		("cat /tmp/mondo-run-prog-thing.err >> /tmp/mondo-run-prog-thing.tmp 2> /dev/null"))
-	{
+	mr_free(callstr);
+
+	mr_asprintf(&callstr, "cat %s/mondo-run-prog-thing.err >> %s/mondo-run-prog-thing.tmp 2> /dev/null", bkpinfo->tmpdir, bkpinfo->tmpdir);
+	if (log_if_failure && system(callstr)) {
 		log_OS_error("Command failed");
 	}
-	unlink("/tmp/mondo-run-prog-thing.err");
-	fin = fopen("/tmp/mondo-run-prog-thing.tmp", "r");
+	mr_free(callstr);
+
+	mr_asprintf(&tmp1, "%s/mondo-run-prog-thing.err", bkpinfo->tmpdir);
+	unlink(tmp1);
+	mr_free(tmp1);
+
+	mr_asprintf(&tmp1, "%s/mondo-run-prog-thing.tmp", bkpinfo->tmpdir);
+	fin = fopen(tmp1, "r");
 	if (fin) {
-		for (fgets(incoming, MAX_STR_LEN, fin); !feof(fin);
-			 fgets(incoming, MAX_STR_LEN, fin)) {
-			/* patch by Heiko Schlittermann */
+		for (q = fgets(incoming, MAX_STR_LEN, fin); !feof(fin) && (q != NULL); q = fgets(incoming, MAX_STR_LEN, fin)) {
 			p = incoming;
 			while (p && *p) {
 				if ((p = strchr(p, '%'))) {
@@ -389,14 +323,15 @@ int run_program_and_log_output(char *program, int debug_level)
 			}
 			/* end of patch */
 			strip_spaces(incoming);
-			if ((res == 0 && log_if_success)
-				|| (res != 0 && log_if_failure)) {
+			if ((res == 0 && log_if_success) || (res != 0 && log_if_failure)) {
 				log_msg(0, incoming);
 			}
 		}
 		paranoid_fclose(fin);
 	}
-	unlink("/tmp/mondo-run-prog-thing.tmp");
+	unlink(tmp1);
+	mr_free(tmp1);
+
 	if ((res == 0 && log_if_success) || (res != 0 && log_if_failure)) {
 		log_msg(0,
 				"--------------------------------end of output------------------------------");
@@ -432,39 +367,45 @@ int run_program_and_log_to_screen(char *basic_call, char *what_i_am_doing)
 	FILE *fin;
 
 	/*@ buffers **************************************************** */
-	char tmp[MAX_STR_LEN * 2];
-	char command[MAX_STR_LEN * 2];
+	char *tmp = NULL;
+	char *command = NULL;
 	char lockfile[MAX_STR_LEN];
 
 	/*@ end vars *************************************************** */
 
 	assert_string_is_neither_NULL_nor_zerolength(basic_call);
 
-	sprintf(lockfile, "/tmp/mojo-jojo.blah.XXXXXX");
-	mkstemp(lockfile);
-	sprintf(command,
+	sprintf(lockfile, "%s/mojo-jojo.bla.bla", bkpinfo->tmpdir);
+
+	mr_asprintf(&command,
 			"echo hi > %s ; %s >> %s 2>> %s; res=$?; sleep 1; rm -f %s; exit $res",
 			lockfile, basic_call, MONDO_LOGFILE, MONDO_LOGFILE, lockfile);
 	open_evalcall_form(what_i_am_doing);
-	sprintf(tmp, "Executing %s", basic_call);
+	mr_asprintf(&tmp, "Executing %s", basic_call);
 	log_msg(2, tmp);
+	mr_free(tmp);
+
 	if (!(fin = popen(command, "r"))) {
 		log_OS_error("Unable to popen-in command");
-		sprintf(tmp, "Failed utterly to call '%s'", command);
+		mr_asprintf(&tmp, "Failed utterly to call '%s'", command);
 		log_to_screen(tmp);
+		mr_free(tmp);
+		mr_free(command);
 		return (1);
 	}
 	if (!does_file_exist(lockfile)) {
-		log_to_screen("Waiting for external binary to start");
+		log_to_screen("Waiting for '%s' to start",command);
 		for (i = 0; i < 60 && !does_file_exist(lockfile); sleep(1), i++) {
 			log_msg(3, "Waiting for lockfile %s to exist", lockfile);
 		}
 	}
+	mr_free(command);
 #ifdef _XWIN
 	/* This only can update when newline goes into the file,
 	   but it's *much* prettier/faster on Qt. */
 	while (does_file_exist(lockfile)) {
 		while (!feof(fin)) {
+			/* TODO: Dead and wrong code */
 			if (!fgets(tmp, 512, fin))
 				break;
 			log_to_screen(tmp);
@@ -513,15 +454,19 @@ int copy_from_src_to_dest(FILE * f_orig, FILE * f_archived, char direction)
 // if dir=='r' then copy from archived to orig
 	char *tmp;
 	char *buf;
+	char filestr[MAX_STR_LEN];
 	long int bytes_to_be_read, bytes_read_in, bytes_written_out =
 		0, bufcap, subsliceno = 0;
 	int retval = 0;
 	FILE *fin;
 	FILE *fout;
 	FILE *ftmp;
+	int tmpcap = 512;
 
 	log_msg(5, "Opening.");
-	malloc_string(tmp);
+	if (!(tmp = malloc(tmpcap))) {
+		fatal_error("Failed to malloc() tmp");
+	}
 	tmp[0] = '\0';
 	bufcap = 256L * 1024L;
 	if (!(buf = malloc(bufcap))) {
@@ -613,16 +558,23 @@ int copy_from_src_to_dest(FILE * f_orig, FILE * f_archived, char direction)
 			}
 			log_msg(5, "tmpB is %s", tmp);
 			if (!strstr(tmp, PIMP_END_SZ)) {
-				ftmp = fopen("/tmp/out.leftover", "w");
+				sprintf(filestr, "%s/out.leftover", bkpinfo->tmpdir);
+				ftmp = fopen(filestr, "w");
 				bytes_read_in = fread(tmp, 1, 64, fin);
 				log_msg(1, "bytes_read_in = %ld", bytes_read_in);
 //      if (bytes_read_in!=128+64) { fatal_error("Can't read the terminating block"); }
-				fwrite(tmp, 1, bytes_read_in, ftmp);
+				if (fwrite(tmp, 1, bytes_read_in, ftmp)) {
+					fatal_error("Can't fwrite here");
+				}
 				sprintf(tmp, "I am here - %lld", (long long)ftello(fin));
 //    log_msg(0, tmp);
-				fread(tmp, 1, 512, fin);
+				if (fread(tmp, 1, tmpcap, fin)) {
+					fatal_error("Can't fread here");
+				}
 				log_msg(0, "tmp = '%s'", tmp);
-				fwrite(tmp, 1, 512, ftmp);
+				if (fwrite(tmp, 1, tmpcap, ftmp)) {
+					fatal_error("Can't fwrite there");
+				}
 				fclose(ftmp);
 				fatal_error("Missing terminating block");
 			}
@@ -657,7 +609,7 @@ int feed_into_ntfsprog(char *input_device, char *output_fname)
 		fatal_error("ntfsclone not found");
 	}
 	malloc_string(command);
-	sprintf(command, "ntfsclone --force --save-image --overwrite %s %s", output_fname, input_device);
+	sprintf(command, "ntfsclone --rescue --force --save-image --overwrite %s %s", output_fname, input_device);
 	res = run_program_and_log_output(command, 5);
 	paranoid_free(command);
 	unlink(output_fname);
@@ -665,77 +617,6 @@ int feed_into_ntfsprog(char *input_device, char *output_fname)
 }
 
 
-
-
-
-int run_external_binary_with_percentage_indicator_OLD(char *tt, char *cmd)
-{
-
-	/*@ int *************************************************************** */
-	int res = 0;
-	int percentage = 0;
-	int maxpc = 0;
-	int pcno = 0;
-	int last_pcno = 0;
-
-	/*@ buffers *********************************************************** */
-	char *command;
-	char *tempfile;
-	char *title;
-	/*@ pointers ********************************************************** */
-	FILE *pin;
-
-	malloc_string(title);
-	malloc_string(command);
-	malloc_string(tempfile);
-	assert_string_is_neither_NULL_nor_zerolength(cmd);
-	assert_string_is_neither_NULL_nor_zerolength(title);
-
-	strcpy(title, tt);
-	strcpy(tempfile,
-		   call_program_and_get_last_line_of_output
-		   ("mktemp -q /tmp/mondo.XXXXXXXX"));
-	sprintf(command, "%s >> %s 2>> %s; rm -f %s", cmd, tempfile, tempfile,
-			tempfile);
-	log_msg(3, command);
-	open_evalcall_form(title);
-	if (!(pin = popen(command, "r"))) {
-		log_OS_error("fmt err");
-		return (1);
-	}
-	maxpc = 100;
-// OLD OLD OLD OLD OLD OLD OLD OLD OLD OLD OLD OLD
-	for (sleep(1); does_file_exist(tempfile); sleep(1)) {
-		pcno = grab_percentage_from_last_line_of_file(MONDO_LOGFILE);
-		if (pcno < 0 || pcno > 100) {
-			log_msg(5, "Weird pc#");
-			continue;
-		}
-		percentage = pcno * 100 / maxpc;
-		if (pcno <= 5 && last_pcno > 40) {
-			close_evalcall_form();
-			strcpy(title, "Verifying...");
-			open_evalcall_form(title);
-		}
-		last_pcno = pcno;
-		update_evalcall_form(percentage);
-	}
-// OLD OLD OLD OLD OLD OLD OLD OLD OLD OLD OLD OLD
-	close_evalcall_form();
-	if (pclose(pin)) {
-		res++;
-		log_OS_error("Unable to pclose");
-	}
-	unlink(tempfile);
-	paranoid_free(command);
-	paranoid_free(tempfile);
-	paranoid_free(title);
-	return (res);
-}
-
-
-
-
 void *run_prog_in_bkgd_then_exit(void *info)
 {
 	char *sz_command;
@@ -845,7 +726,7 @@ int feed_outfrom_ntfsprog(char *output_device, char *input_fifo)
 		fatal_error("ntfsclone not found");
 	}
 	malloc_string(command);
-	sprintf(command, "ntfsclone --force --restore-image --overwrite %s %s", output_device, input_fifo);
+	sprintf(command, "ntfsclone --rescue --force --restore-image --overwrite %s %s", output_device, input_fifo);
 	res = run_program_and_log_output(command, 5);
 	paranoid_free(command);
 	return (res);
