@@ -1,4 +1,4 @@
-/* $Id: libmondo-string.c 1236 2007-03-11 23:23:16Z bruno $ */
+/* $Id: libmondo-string.c 3150 2013-06-21 04:48:23Z bruno $ */
 
 
 /**
@@ -7,6 +7,7 @@
  */
 
 #include "my-stuff.h"
+#include "mr_mem.h"
 #include "mondostructures.h"
 #include "libmondo-string.h"
 #include "lib-common-externs.h"
@@ -15,11 +16,14 @@
 #include "libmondo-tools-EXT.h"
 
 /*@unused@*/
-//static char cvsid[] = "$Id: libmondo-string.c 1236 2007-03-11 23:23:16Z bruno $";
+//static char cvsid[] = "$Id: libmondo-string.c 3150 2013-06-21 04:48:23Z bruno $";
 
 extern int g_current_media_number;
 extern long long g_tape_posK;
 
+/* Reference to global bkpinfo */
+extern struct s_bkpinfo *bkpinfo;
+
 
 /**
  * @addtogroup stringGroup
@@ -217,14 +221,13 @@ long friendly_sizestr_to_sizelong(char *incoming)
 
 	assert_string_is_neither_NULL_nor_zerolength(incoming);
 
-	malloc_string(tmp);
 	if (!incoming[0]) {
-		free(tmp);
 		return (0);
 	}
 	if (strchr(incoming, '.')) {
 		fatal_error("Please use integers only. No decimal points.");
 	}
+	malloc_string(tmp);
 	strcpy(tmp, incoming);
 	i = (int) strlen(tmp);
 	if (tmp[i - 1] == 'B' || tmp[i - 1] == 'b') {
@@ -397,7 +400,7 @@ char *mountlist_entry_to_string(struct mountlist_itself *mountlist,
 
 	sprintf(output, "%-24s %-24s %-10s %8lld", mountlist->el[lino].device,
 			mountlist->el[lino].mountpoint, mountlist->el[lino].format,
-			mountlist->el[lino].size / 1024);
+			mountlist->el[lino].size / 1024L);
 	return (output);
 }
 
@@ -595,60 +598,6 @@ int special_dot_char(int i)
 }
 
 
-
-
-/**
- * Wrap @p flaws_str across three lines. The first two are no more than 74 characters wide.
- * @param flaws_str The original string to split.
- * @param flaws_str_A Where to put the first 74-or-less characters.
- * @param flaws_str_B Where to put the second 74-or-less characters.
- * @param flaws_str_C Where to put the rest.
- * @param res The result of the original evaluate_mountlist() operation.
- * @return TRUE if res == 0, FALSE otherwise.
- */
-bool
-spread_flaws_across_three_lines(char *flaws_str, char *flaws_str_A,
-								char *flaws_str_B, char *flaws_str_C,
-								int res)
-{
-
-	/*@ int ************************************************************* */
-	int i = 0;
-
-	/*@ initialize ****************************************************** */
-	assert(flaws_str_A != NULL);
-	assert(flaws_str_B != NULL);
-	assert(flaws_str_C != NULL);
-	assert(flaws_str != NULL);
-
-	flaws_str_A[0] = flaws_str_B[0] = flaws_str_C[0] = '\0';
-
-
-	if (!res && !strlen(flaws_str)) {
-		return (TRUE);
-	}
-	if (strlen(flaws_str) > 0) {
-		sprintf(flaws_str_A, "%s", flaws_str + 1);
-	}
-	if (strlen(flaws_str_A) >= 74) {
-		for (i = 74; flaws_str_A[i] != ' '; i--);
-		strcpy(flaws_str_B, flaws_str_A + i + 1);
-		flaws_str_A[i] = '\0';
-	}
-	if (strlen(flaws_str_B) >= 74) {
-		for (i = 74; flaws_str_B[i] != ' '; i--);
-		strcpy(flaws_str_C, flaws_str_B + i + 1);
-		flaws_str_B[i] = '\0';
-	}
-	if (res) {
-		return (FALSE);
-	} else {
-		return (TRUE);
-	}
-}
-
-
-
 /**
  * Compare @p stringA and @p stringB. This uses an ASCII sort for everything
  * up to the digits on the end but a numerical sort for the digits on the end.
@@ -744,6 +693,34 @@ char *strip_afio_output_line(char *input)
  * @param in_out The string to strip spaces/control characters from (modified).
  */
 void strip_spaces(char *in_out)
+{
+	/*@ buffers ***************************************************** */
+	char *tmp = NULL;
+
+	/*@ int ******************************************************** */
+	int i;
+
+	/*@ end vars *************************************************** */
+
+	assert(in_out != NULL);
+	malloc_string(tmp);
+	for (i = 0; in_out[i] <= ' ' && i < (int) strlen(in_out) -1; i++);
+	strcpy(tmp, in_out + i);
+	for (i = (int) strlen(tmp) -1; i >= 0 && tmp[i] <= ' '; i--);
+	i++;
+	tmp[i] = '\0';
+
+	// Now tmp contains the stripped string
+	strcpy(in_out,tmp);
+	paranoid_free(tmp);
+}
+
+/**
+ * Remove all characters whose ASCII value is less than or equal to 32
+ * (spaces and control characters) from both sides of @p in_out.
+ * @param in_out The string to strip spaces/control characters from (modified).
+ */
+void strip_spaces2(char *in_out)
 {
 	/*@ buffers ***************************************************** */
 	char *tmp;
@@ -832,43 +809,60 @@ char *trim_empty_quotes(char *incoming)
  * @param partition The partition name soon-to-become drive name. (modified)
  * @return @p partition.
  */
-char *truncate_to_drive_name(char *partition)
+char *truncate_to_drive_name(const char *partition)
 {
 	int i = strlen(partition) - 1;
 	char *c;
+	char *trunc = NULL;
+
+	assert_string_is_neither_NULL_nor_zerolength(partition);
+	mr_asprintf(&trunc, "%s", partition);
 
 #ifdef __FreeBSD__
 
-	if (islower(partition[i]))	// BSD subpartition
+	if (islower(trunc[i]))	// BSD subpartition
 		i--;
-	if (partition[i - 1] == 's') {
-		while (isdigit(partition[i]))
+	if (trunc[i-1] == 's') {
+		while (isdigit(trunc[i]))
 			i--;
 		i--;
 	}
-	partition[i + 1] = '\0';
+	trunc[i+1] = '\0';
 
 #else
 
-	assert_string_is_neither_NULL_nor_zerolength(partition);
 	/* first see if it's a devfs style device */
-	c = strrchr(partition, '/');
+	c = strrchr(trunc, '/');
 	if (c && strncmp(c, "/part", 5) == 0) {
 		/* yup it's devfs, return the "disc" path */
 		strcpy(c + 1, "disc");
-		return partition;
+		return trunc;
+	}
+	/* then see if it's a dm style device */
+	if (c && strncmp(c, "/dm-", 4) == 0) {
+		/* yup it's dm, return the full path */
+		return trunc;
 	}
 
-	for (i = strlen(partition); isdigit(partition[i - 1]); i--)
+
+	for (i = strlen(trunc); isdigit(trunc[i-1]); i--)
 		continue;
-	if (partition[i - 1] == 'p' && isdigit(partition[i - 2])) {
+	if (trunc[i-1] == 'p' && isdigit(trunc[i-2])) {
 		i--;
+	} else {
+		/* Some full devices like this /dev/mapper/mpath0
+		   /dev/cciss/c0d0 may be used as partition names */
+		if ((strstr(trunc,"/dev/mapper/mpath") != NULL) ||
+			(strstr(trunc,"/dev/cciss/c") != NULL) ||
+			(strstr(trunc,"/dev/rd/") != NULL)) {
+				return trunc;
+			}
 	}
-	partition[i] = '\0';
+	trunc[i] = '\0';
 
 #endif
 
-	return partition;
+	return trunc;
 }
 
 
@@ -943,7 +937,7 @@ int severity_of_difference(char *fn, char *out_reason)
 	if (!strncmp(filename, "/home", 5)) {
 		sev = 2;
 		sprintf(reason,
-				"It's in your /home partiton. Therefore, it is important.");
+				"It's in your /home directory. Therefore, it is important.");
 	}
 	if (!strncmp(filename, "/usr/", 5)) {
 		sev = 3;
@@ -1061,7 +1055,7 @@ int compare_two_filelist_entries(void *va, void *vb)
  * @return The string indicating media fill.
  * @note The returned string points to static storage that will be overwritten with each call.
  */
-char *percent_media_full_comment(struct s_bkpinfo *bkpinfo)
+char *percent_media_full_comment()
 {
 	/*@ int *********************************************** */
 	int percentage;
@@ -1070,6 +1064,7 @@ char *percent_media_full_comment(struct s_bkpinfo *bkpinfo)
 	/*@ buffers ******************************************* */
 	static char outstr[MAX_STR_LEN];
 	char *pos_w_commas, *tmp;
+	char *mds = NULL;
 
 	assert(bkpinfo != NULL);
 	malloc_string(pos_w_commas);
@@ -1079,7 +1074,7 @@ char *percent_media_full_comment(struct s_bkpinfo *bkpinfo)
 
 
 
-	if (bkpinfo->media_size[g_current_media_number] <= 0)
+	if (bkpinfo->media_size <= 0)
 //    { fatal_error( "percentage_media_full_comment() - unknown media size"); }
 	{
 		sprintf(outstr, "Volume %d: %s kilobytes archived so far",
@@ -1089,20 +1084,17 @@ char *percent_media_full_comment(struct s_bkpinfo *bkpinfo)
 
 /* update screen */
 	if (IS_THIS_A_STREAMING_BACKUP(bkpinfo->backup_media_type)) {
-		percentage =
-			(int) (g_tape_posK / 10 /
-				   bkpinfo->media_size[g_current_media_number]);
+		percentage = (int) (g_tape_posK / 10 / bkpinfo->media_size);
 		if (percentage > 100) {
 			percentage = 100;
 		}
 		sprintf(outstr, "Volume %d: [", g_current_media_number);
 	} else {
 		percentage =
-			(int) (space_occupied_by_cd(bkpinfo->scratchdir) * 100 / 1024 /
-				   bkpinfo->media_size[g_current_media_number]);
-		sprintf(outstr, "%s %d: [",
-				media_descriptor_string(bkpinfo->backup_media_type),
-				g_current_media_number);
+			(int) (space_occupied_by_cd(bkpinfo->scratchdir) * 100 / 1024 / bkpinfo->media_size);
+		mds = media_descriptor_string(bkpinfo->backup_media_type);
+		sprintf(outstr, "%s %d: [", mds, g_current_media_number);
+		mr_free(mds);
 	}
 	for (j = 0; j < percentage; j += 5) {
 		strcat(outstr, "*");
@@ -1123,41 +1115,40 @@ char *percent_media_full_comment(struct s_bkpinfo *bkpinfo)
  * @return The stringification of @p type_of_bkp.
  * @note The returned string points to static storage that will be overwritten with each call.
  */
-char *media_descriptor_string(t_bkptype type_of_bkp)
-{
-	static char *type_of_backup = NULL;
+char *media_descriptor_string(t_bkptype type_of_bkp) {
 
-	if (!type_of_backup) {
-		malloc_string(type_of_backup);
-	}
+	char *type_of_backup = NULL;
 
 	switch (type_of_bkp) {
 	case dvd:
-		strcpy(type_of_backup, "DVD");
+		mr_asprintf(&type_of_backup, "DVD");
 		break;
 	case cdr:
-		strcpy(type_of_backup, "CDR");
+		mr_asprintf(&type_of_backup, "CDR");
 		break;
 	case cdrw:
-		strcpy(type_of_backup, "CDRW");
+		mr_asprintf(&type_of_backup, "CDRW");
 		break;
 	case tape:
-		strcpy(type_of_backup, "tape");
+		mr_asprintf(&type_of_backup, "tape");
 		break;
 	case cdstream:
-		strcpy(type_of_backup, "CDR");
+		mr_asprintf(&type_of_backup, "CDR");
 		break;
 	case udev:
-		strcpy(type_of_backup, "udev");
+		mr_asprintf(&type_of_backup, "udev");
 		break;
 	case iso:
-		strcpy(type_of_backup, "ISO");
+		mr_asprintf(&type_of_backup, "ISO");
 		break;
-	case nfs:
-		strcpy(type_of_backup, "nfs");
+	case netfs:
+		mr_asprintf(&type_of_backup, "%s", bkpinfo->netfs_proto);
 		break;
+    case usb:
+        mr_asprintf(&type_of_backup, "USB");
+        break;
 	default:
-		strcpy(type_of_backup, "ISO");
+		mr_asprintf(&type_of_backup, "ISO");
 	}
 	return (type_of_backup);
 }
