@@ -1,10 +1,11 @@
 	/* libmondo-tools.h
-	 * $Id: libmondo-tools.h 1375 2007-04-30 09:18:15Z bruno $
+	 * $Id: libmondo-tools.h 2470 2009-11-10 11:32:40Z bruno $
 	 */
 
 void clean_up_KDE_desktop_if_necessary(void);
 
 long get_time();
+char *mr_date(void);
 extern void (*log_debug_msg) (int debug_level, const char *szFile,
 							  const char *szFunction, int nLine,
 							  const char *fmt, ...);
@@ -13,13 +14,12 @@ void standard_log_debug_msg(int debug_level, const char *szFile,
 							const char *fmt, ...);
 int read_cfg_var(char *config_file, char *label, char *value);
 int write_cfg_var(char *config_file, char *label, char *value);
-void reset_bkpinfo(struct s_bkpinfo *bkpinfo);
+void reset_bkpinfo();
 #ifdef __FreeBSD__
 void initialize_raidrec(struct vinum_volume *vv);
 #else
 void initialize_raidrec(struct raid_device_record *raidrec);
 #endif
-void log_trace(char *o);
 int some_basic_system_sanity_checks();
 
 
@@ -30,10 +30,10 @@ int find_and_store_mondoarchives_home(char *home_sz);
 void unmount_supermounts_if_necessary(void);
 void remount_supermounts_if_necessary(void);
 
-int post_param_configuration(struct s_bkpinfo *bkpinfo);
+int post_param_configuration();
 
 
-int pre_param_configuration(struct s_bkpinfo *bkpinfo);
+int pre_param_configuration();
 
 long free_space_on_given_partition(char *partition);
 
